/* eslint-disable react/jsx-props-no-spreading */
import React, { FC, useEffect, useState } from 'react';
import TextField from '@material-ui/core/TextField';
import Autocomplete from '@material-ui/lab/Autocomplete';
import { getSeniorListByLastName } from 'api/getSeniorListByLastName';
import { SeniorSearchQueryType } from 'types/SeniorSearchQueryType';
import { getSeniorListByCard } from 'api/getSeniorListByCard';

type PropType = {
  senior: SeniorSearchQueryType | null;
  setSenior: React.Dispatch<React.SetStateAction<SeniorSearchQueryType | null>>;
};

/**
 * Search a senior by member card or by last name.
 * @param {PropType} react action data for connecting this component.
 * @returns
 */
export const SeniorSearch: FC<PropType> = ({ senior, setSenior }: PropType) => {
  const [seniors, setSeniors] = useState<SeniorSearchQueryType[]>([]);
  const [query, setQuery] = useState<string | number>('');

  useEffect(() => {
    const MIN_SEARCH_LENGTH = 4;
    if (typeof query === 'string') {
      if (query.length >= MIN_SEARCH_LENGTH) {
        getSeniorListByLastName(query).then((list) => {
          setSeniors(list);
        });
      }
    } else if (typeof query === 'number') {
      getSeniorListByCard(query).then((list) => {
        setSeniors(list);
      });
    }
  }, [query]);

  const handleChange = (newValue: string): void => {
    const cardNumber = Number(newValue);
    if (Number.isNaN(cardNumber)) {
      setQuery(newValue);
    } else {
      setQuery(cardNumber);
    }
  };

  return (
    <Autocomplete
      value={senior}
      id="senior-searcher"
      onInputChange={(_, newValue) => handleChange(newValue)}
      onChange={(_, val) => {
        setSenior(val);
      }}
      options={seniors}
      getOptionSelected={(option, val) => option.id === val.id}
      /**
       * Filter by last name if the query is string, otherwise use the card number.
       */
      getOptionLabel={(option) =>
        typeof query === 'string'
          ? option.user.lastName
          : option.memberCardNumber.toString()
      }
      renderOption={(option) => (
        <>
          {option.user.firstName} {option.user.lastName} {option.user.username}{' '}
          {option.memberCardNumber}
        </>
      )}
      renderInput={(params) => (
        <TextField
          {...params}
          label="Search a senior"
          variant="outlined"
          InputProps={{
            ...params.InputProps,
          }}
        />
      )}
    />
  );
};
